/* stb_image implementation — PNG-only build for kitty graphics.
 *
 * Vendored from: https://github.com/nothings/stb
 * Version:       v2.30 (see header comment in vendor/stb/stb_image.h)
 * Vendored:      2026-04 */
#define STB_IMAGE_IMPLEMENTATION
#define STBI_ONLY_PNG
#define STBI_NO_STDIO
#define STBI_NO_LINEAR
#define STBI_NO_HDR
#include "stb_image.h"
